CON
           
' *******************************************************************************************************
' *                                                                                                     *
' *     AiChip I2C Logger Demonstration                                  AiChip_I2cLogger_Demo_001      *
' *                                                                                                     *
' *******************************************************************************************************
' *                                                                                                     *
' *     Object Name    : AiChip I2C LoggerDemonstration                                                 *
' *     Version        : 001                                                                            *
' *     Updated        : 2008-04-12                                                                     *
' *                                                                                                     *
' *     Author         : AiChip Industries                                                              *
' *                                                                                                     *
' *     Copyright      : 2008, AiChip Industries                                                        *
' *                                                                                                     *
' *     Target Chip    : Parallax Propeller Chip P8X32A ( "Propeller Mk I" )                            *
' *     Compiler Used  : Parallax Propeller Tool 1.1                                                    *
' *                                                                                                     *
' *     License        : MIT License. Please see end of this file for Terms of Use.                     *
' *                                                                                                     *
' *******************************************************************************************************
' *                                                                                                     *
' *     AiChip is a trademark of AiChip Industries.                                                     *
' *                                                                                                     *
' *     If distributing a modified version of this software AiChip Industries requests the "AiChip"     *
' *     prefix be removed from the filename and internal text descriptions to avoid any confusion that  *
' *     may be caused by an impression that the modified software originates from AiChip Industries.    *
' *                                                                                                     *
' *******************************************************************************************************

PRI Version

  result := String( "AiChip_I2cLogger_Demo_001" )
  
CON

  _CLKMODE      = XTAL1 + PLL16x
  _XINFREQ      = 5_000_000

  TV_PIN        = 12
  
  SDA_PIN       = 29
  SCL_PIN       = 28

OBJ

  tv            : "TV_Text"
  
  i2c_Slave     : "AiChip_I2cSlave_Logger_001"
  i2c_Master    : "AiChip_I2cSlave_Master_003"
  
PUB Demo | logPtr

  tv.Start( TV_PIN )

  ' Start the I2C Slave running. It uses the same I2C lines as the master does so no wiring needed.

  i2c_Slave.Start( SCL_PIN, SDA_PIN, $E0, 0, 0, 0 )

  ' Allow the I2C Slave time to start so we can be sure that starting the I2C Master doesn't cause it any
  ' problems when kicking the I2C bus into life.
  
  WaitCnt( CLKFREQ / 1000 * 100 + CNT )

  ' Initialise the I2C Master code so it knows what it's about to communicate with.
  
  i2c_Master.OpenRaw( SCL_PIN, SDA_PIN, $E0, 0, 0 )

  i2c_Master.WriteByte($1234,$56)
  i2c_Master.ReadByte($1234)

  logPtr := i2c_Slave.AtLogData

  ' Logged data is ...          1111 1111 1111 1111 1111 1111 1111 1111     Not used
  '                             0000 0000 0000 0000 0000 0000 0nnn nnnn     Character "S", "0", "1", "E"
  '                             0000 0000 0000 0000 0000 0000 0000 00cd     Sclk / Sda data bits
  '                             1000 0000 0000 0000 0000 0000 nnnn nnnn     Byte data
  '                             1100 0000 0000 0000 nnnn nnnn nnnn nnnn     Word data

  repeat                                
    repeat while ( result := long[ logPtr ] ) == -1
 
    if ( result & $FFFF_FF00 ) == $0000_0000
      if ( result & $FFFF_FFFC ) == $0000_0000
        ' tv.Out( " " )
        ' tv.Bin( result, 2 ) 
      else
        if result == "S"
          tv.Out( $0D )
        tv.Out( result )

    if ( result & $C000_0000 ) == $8000_0000
      tv.Out( " " )
      tv.Hex( result, 2 ) 

    if ( result & $C000_0000 ) == $C000_0000
      tv.Out( " " )
      tv.Hex( result, 4 ) 

    logPtr += 4
    

CON

' *******************************************************************************************************
' *                                                                                                     *
' *     Terms of Use : MIT License                                                                      *
' *                                                                                                     *
' *******************************************************************************************************
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
' associated documentation files (the "Software"), to deal in the Software without restriction, including
' without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
' copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to
' the following conditions:                                                                   
'
' The above copyright notice and this permission notice shall be included in all copies or substantial
' portions of the Software.                                                                                             
'                                                                                                                  
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
' LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
' NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
' WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
' SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

CON

' *******************************************************************************************************
' *                                                                                                     *
' *     End of AiChip I2C Slave Demonstration                                                           *
' *                                                                                                     *
' *******************************************************************************************************
        