CON
           
' *******************************************************************************************************
' *                                                                                                     *
' *     AiChip I2C Logger Interface                                      AiChip_I2cSlave_Logger_001     *
' *                                                                                                     *
' *******************************************************************************************************
' *                                                                                                     *
' *     Object Name    : AiChip I2C Logger Interface                                                    *
' *     Version        : 001                                                                            *
' *     Updated        : 2008-04-12                                                                     *
' *                                                                                                     *
' *     Author         : AiChip Industries                                                              *
' *                                                                                                     *
' *     Copyright      : 2008, AiChip Industries                                                        *
' *                                                                                                     *
' *     Target Chip    : Parallax Propeller Chip P8X32A ( "Propeller Mk I" )                            *
' *     Compiler Used  : Parallax Propeller Tool 1.1                                                    *
' *                                                                                                     *
' *     License        : MIT License. Please see end of this file for Terms of Use.                     *
' *                                                                                                     *
' *******************************************************************************************************
' *                                                                                                     *
' *     AiChip is a trademark of AiChip Industries.                                                     *
' *                                                                                                     *
' *     If distributing a modified version of this software AiChip Industries requests the "AiChip"     *
' *     prefix be removed from the filename and internal text descriptions to avoid any confusion that  *
' *     may be caused by an impression that the modified software originates from AiChip Industries.    *
' *                                                                                                     *
' *******************************************************************************************************

PUB Version

  result := String( "AiChip_I2cSlave_Logger_001" )

CON

  LOG_SIZE = 2000
  
VAR

  byte CogNumber

  long logData[ LOG_SIZE+2 ]
  
PUB Start( sclPin, sdaPin, id, a2, a1, a0 )

  Stop

  id := ( id & $FE ) | a2 << 3 | a1 << 2 | a0 << 1

  a2 := @logData
  a1 := LOG_SIZE

  repeat a0 from 0 to LOG_SIZE+1
    logData[ a0 ] := -1
    
  cogNumber := CogNew( @I2cSlave, @sclPin ) + 1

  repeat while sclPin => 0
  
PUB Stop

  if cogNumber
    CogStop( cogNumber~ - 1 )

PUB AtLogData

  result := @logData
  
CON

' *******************************************************************************************************
' *                                                                                                     *
' *     PASM Handler                                                                                    *
' *                                                                                                     *
' *******************************************************************************************************
                        
DAT                     org     $000

'                       .-------------------------------------------------------------------------------.
'                       |       initialise the parameters                                               |
'                       `-------------------------------------------------------------------------------'

I2cSlave                mov     idx,PAR

GetParam                rdlong  byt,idx
                        add     idx,#4
GetParam_Ret            jmp     #FirstParam

FirstParam              mov     sclMask,#1
                        shl     sclMask,byt                                   

                        call    #GetParam                                       
                        mov     sdaMask,#1
                        shl     sdaMask,byt                                   

                        mov     sxxMask,sclMask
                        or      sxxMask,sdaMask
                        
                        call    #GetParam                                       
                        and     byt,#$FE
                        mov     deviceWr,byt
                        or      byt,#1
                        mov     deviceRd,byt

                        call    #GetParam
                        mov     logPtr,byt

                        call    #GetParam
                        mov     logCnt,byt
                        
'                       .-------------------------------------------------------------------------------.
'                       |       Tell caller the cog is running                                          |
'                       `-------------------------------------------------------------------------------'

                        neg     byt,#1
                        wrlong  byt,PAR

'                       .-------------------------------------------------------------------------------.
'                       |       Determine bit or event from the incoming bit stream                     |
'                       `-------------------------------------------------------------------------------'
'
GetBit                  call    #GetBitStream           ' Match the bit stream

                        mov     idx,bitStream
                        add     idx,#JumpTable
                        jmp     idx
                        
JumpTable               jmp     #GetBit                 ' 00_00_00
                        jmp     #GetBit                 ' 00_00_01
                        jmp     #GetBit                 ' 00_00_10
                        jmp     #GetBit                 ' 00_00_11
                        jmp     #GetBit                 ' 00_01_00              
                        jmp     #GetBit                 ' 00_01_01
                        jmp     #GetBit                 ' 00_01_10
                        jmp     #GetBit                 ' 00_01_11               
                        jmp     #Bit0                   ' 00_10_00              Bit 0
                        jmp     #GetBit                 ' 00_10_01
                        jmp     #GetBit                 ' 00_10_10
                        jmp     #StopBit                ' 00_10_11              Stop Bit
                        jmp     #GetBit                 ' 00_11_00
                        jmp     #GetBit                 ' 00_11_01
                        jmp     #GetBit                 ' 00_11_10
                        jmp     #GetBit                 ' 00_11_11
                        
                        jmp     #GetBit                 ' 01_00_00
                        jmp     #GetBit                 ' 01_00_01             
                        jmp     #getBit                 ' 01_00_10           
                        jmp     #GetBit                 ' 01_00_11
                        jmp     #GetBit                 ' 01_01_00
                        jmp     #GetBit                 ' 01_01_01
                        jmp     #GetBit                 ' 01_01_10
                        jmp     #GetBit                 ' 01_01_11
                        jmp     #GetBit                 ' 01_10_00
                        jmp     #GetBit                 ' 01_10_01
                        jmp     #GetBit                 ' 01_10_10
                        jmp     #GetBit                 ' 01_10_11
                        jmp     #GetBit                 ' 01_11_00
                        jmp     #Bit1                   ' 01_11_01              Bit 1
                        jmp     #GetBit                 ' 01_11_10              
                        jmp     #GetBit                 ' 01_11_11
                        
                        jmp     #GetBit                 ' 10_00_00
                        jmp     #GetBit                 ' 10_00_01             
                        jmp     #GetBit                 ' 10_00_10          
                        jmp     #GetBit                 ' 10_00_11
                        jmp     #GetBit                 ' 10_01_00
                        jmp     #GetBit                 ' 10_01_01
                        jmp     #GetBit                 ' 10_01_10
                        jmp     #GetBit                 ' 10_01_11
                        jmp     #GetBit                 ' 10_10_00
                        jmp     #GetBit                 ' 10_10_01
                        jmp     #GetBit                 ' 10_10_10
                        jmp     #GetBit                 ' 10_10_11
                        jmp     #GetBit                 ' 10_11_00
                        jmp     #GetBit                 ' 10_11_01              
                        jmp     #GetBit                 ' 10_11_10              
                        jmp     #GetBit                 ' 10_11_11
                        
                        jmp     #GetBit                 ' 11_00_00
                        jmp     #GetBit                 ' 11_00_01
                        jmp     #GetBit                 ' 11_00_10
                        jmp     #GetBit                 ' 11_00_11
                        jmp     #GetBit                 ' 11_01_00              
                        jmp     #GetBit                 ' 11_01_01
                        jmp     #GetBit                 ' 11_01_10
                        jmp     #GetBit                 ' 11_01_11              
                        jmp     #StartBit               ' 11_10_00              Start Bit
                        jmp     #GetBit                 ' 11_10_01
                        jmp     #GetBit                 ' 11_10_10
                        jmp     #GetBit                 ' 11_10_11              
                        jmp     #GetBit                 ' 11_11_00
                        jmp     #GetBit                 ' 11_11_01
                        jmp     #GetBit                 ' 11_11_10
                        jmp     #GetBit                 ' 11_11_11

StartBit                mov     logDat,#"S"
                        call    #Log
                        jmp     #GetBit

StopBit                 mov     logDat,#"E"
                        call    #Log
                        jmp     #GetBit

Bit0                    mov     logDat,#"0"
                        call    #Log
                        jmp     #GetBit

Bit1                    mov     logDat,#"1"
                        call    #Log
                        jmp     #GetBit

'                       .-------------------------------------------------------------------------------.
'                       |       Read a bit pair into the bit stream                                     |
'                       `-------------------------------------------------------------------------------'

GetBitStream            mov     thisInput,INA           ' Read SCLK/SDA
                        and     thisInput,sxxMask
                        cmp     thisInput,lastInput WZ  ' Same as last time ?
                  IF_Z  jmp     #GetBitStream           ' Yes - Wait for change

                        mov     lastInput,thisInput                             

                        test    thisInput,sclMask WC    ' Update bit stream
                        rcl     bitStream,#1
                        test    thisInput,sdaMask WC
                        rcl     bitStream,#1
                        and     bitStream,#%11_11_11    ' Keep three samples

                        mov     logDat,bitStream
                        and     logDat,#%11
                        call    #Log
                        
GetBitStream_Ret        ret

'                       .-------------------------------------------------------------------------------.
'                       |       Log Data                                                                |
'                       `-------------------------------------------------------------------------------'

Log                     wrlong  logDat,logPtr
                        add     logPtr,#4
                        sub     logCnt,#1
                        tjz     logCnt,#$
Log_Ret                 ret

'                       .-------------------------------------------------------------------------------.
'                       |       Pre-Initialised Variables                                               |
'                       `-------------------------------------------------------------------------------'

lastInput               long    %00

'                       .-------------------------------------------------------------------------------.
'                       |       Variables                                                               |
'                       `-------------------------------------------------------------------------------'

sclMask                 res     1
sdaMask                 res     1
sxxMask                 res     1

deviceRd                res     1
deviceWr                res     1

thisInput               res     1
bitStream               res     1

bitCount                res     1
bit                     res     1
byt                     res     1

adr                     res     1
idx                     res     1

logDat                  res     1
logPtr                  res     1
logCnt                  res     1

                        fit     $1F0

CON

' *******************************************************************************************************
' *                                                                                                     *
' *     Terms of Use : MIT License                                                                      *
' *                                                                                                     *
' *******************************************************************************************************
'
' Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
' associated documentation files (the "Software"), to deal in the Software without restriction, including
' without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
' copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to
' the following conditions:                                                                   
'
' The above copyright notice and this permission notice shall be included in all copies or substantial
' portions of the Software.                                                                                             
'                                                                                                                  
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
' LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
' NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
' WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
' SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

CON

' *******************************************************************************************************
' *                                                                                                     *
' *     End of AiChip I2C Slave Interface                                                               *
' *                                                                                                     *
' *******************************************************************************************************
  